HC = require 'lib.HardonCollider'

--------------------------------------------------------------------------------
function love.load()
	print("LOAD!")

	math.randomseed(os.time())
	math.random();math.random();math.random()

	camera = {speed = 80, distance = 0}
	gravity = 400

	HC.init(100)
	HC.setCallbacks(on_collision, end_collision)

	ball = new_ball()
	ball.image = love.graphics.newImage('img/ball.png')

	wall_left = HC.addRectangle(-5000,-5000, 5000, 15000)
	HC.setPassive(wall_left)
	wall_left.id = "wall"

	wall_right = HC.addRectangle(800,-5000, 5000, 15000)
	HC.setPassive(wall_right)
	wall_right.id = "wall"

	ground = HC.addRectangle(0, 600, 800, 100)
	HC.setPassive(ground)
	ground.id = "ground"

	ceiling = HC.addRectangle(0, -100, 800, 100)
	HC.setPassive(ceiling)
	ceiling.id = "ceiling"

	barriers = {}
	barriers.time = 0
	barriers.default_time = 6
end

--------------------------------------------------------------------------------
function love.update(dt)
	if dt>1/30 then
		dt=1/30
	end
	camera.distance = camera.distance - camera.speed*dt
	ground:move(0, camera.speed*dt)
	wall_left:move(0, camera.speed*dt)
	wall_right:move(0, camera.speed*dt)
	ceiling:move(0, camera.speed*dt)
	camera.speed = camera.speed + camera.speed*0.008*dt
	ball.update(dt)
	barriers.time = barriers.time - dt
	if barriers.time <= 0 then
		table.insert(barriers, new_barrier())
		barriers.time = barriers.default_time
		barriers.default_time =
			math.max(2, barriers.default_time - barriers.default_time*0.05)
		print(barriers.default_time)
	end
	HC.update(dt)
end

--------------------------------------------------------------------------------
function love.draw()
	love.graphics.push()
		love.graphics.translate(0, camera.distance)
		for _,barrier in ipairs(barriers) do
			barrier[1]:draw('fill')
			barrier[2]:draw('fill')
		end
		local ball_x, ball_y = ball:center()
		local ball_a = ball:rotation()
		love.graphics.draw(ball.image, ball_x, ball_y, ball_a, 1, 1, 20, 20)
	love.graphics.pop()
	love.graphics.print(math.floor(math.abs(camera.distance)), 0,0)
end

--------------------------------------------------------------------------------
function on_collision(dt, shape_a, shape_b, mtv_x, mtv_y)
	if shape_a.id == "ball" then
		other = shape_b
	elseif shape_b.id == "ball" then
		other = shape_a
	else
		return
	end
	if other.id == "ceiling" then
		print("YOU LOSE!")
		love.load()
	elseif other.id == "wall" then
		ball.velocity.x = -ball.velocity.x
		ball:move(mtv_x, mtv_y)
	elseif other.id == "ground" then
		ball:move(mtv_x, mtv_y)
		ball:move(0, camera.speed*dt)
		ball.velocity.y = 0
	elseif other.id == "barrier" then
		if not ball.onGround and mtv_y < 0 then
			ball.onGround = true
		end
		if ball.onGround then
			ball.velocity.a = ball.velocity.x
		end
		if mtv_y < 0 then
			ball.velocity.y = 0
		elseif mtv_x ~= 0 then
			ball.velocity.x = 0
		end
		ball:move(mtv_x, mtv_y)
	end
end

--------------------------------------------------------------------------------
function end_collision(dt, shape_a, shape_b)
	if shape_a.id == "ball" or shape_b.id == "ball" then
		other = shape_b
	elseif shape_b.id == "ball" then
		other = shape_a
	else
		return
	end
	if other.id=='barrier' then
		ball.onGround = false
	end
end




--------------------------------------------------------------------------------
function new_ball()
	if ball then
		HC.remove(ball)
	end
	local ball = HC.addCircle(400,300,20)
	ball:moveTo(400, 50)
	ball.id = "ball"
	ball.collisions = 0
	ball.onGround = false
	ball.velocity = {x=0, y=0, a=0}
	ball.update =
		function(dt)
			if not ball.onGround then
				ball.velocity.y = ball.velocity.y + gravity*dt
				ball:move(0, ball.velocity.y*dt)
			end
			ball.velocity.x = ball.velocity.x-0.5*ball.velocity.x*dt
			ball:rotate(ball.velocity.a/(20*20*math.pi))
			if love.keyboard.isDown('left') then
				ball.velocity.x = ball.velocity.x - 600*dt
			elseif love.keyboard.isDown('right') then
				ball.velocity.x = ball.velocity.x + 600*dt
			end
			ball:move(ball.velocity.x*dt, 0)
		end
	return ball
end

--------------------------------------------------------------------------------
function new_barrier()
	local barrier = {}
	local hole_width = 80
	local random_x = math.random(1,799-hole_width)
	local rect = HC.addRectangle(0, -camera.distance + 600, random_x, 30)
	HC.setPassive(rect)
	rect.id = "barrier"
	table.insert(barrier, rect)
	local rect = HC.addRectangle(random_x+hole_width, -camera.distance + 600, 800-(random_x+hole_width), 30)
	HC.setPassive(rect)
	rect.id = "barrier"
	table.insert(barrier, rect)
	return barrier
end

--------------------------------------------------------------------------------
function clean_up()
	--remove physics objects
	love.load()
end
